@@ -1,4 +1,6 @@
-require 'jbuilder/jbuilder'
+# frozen_string_literal: true
+
+require 'jbuilder/version'
 
 class Jbuilder
   class NullError < ::NoMethodError
@@ -14,4 +16,11 @@ class Jbuilder
       new(message)
     end
   end
+
+  class MergeError < ::StandardError
+    def self.build(current_value, updates)
+      message = "Can't merge #{updates.inspect} into #{current_value.inspect}"
+      new(message)
+    end
+  end
 end
