@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'jbuilder/jbuilder'
 require 'jbuilder/collection_renderer'
 require 'action_dispatch/http/mime_type'
@@ -10,10 +12,11 @@ class JbuilderTemplate < Jbuilder
 
   self.template_lookup_options = { handlers: [:jbuilder] }
 
-  def initialize(context, *args)
+  def initialize(context, options = nil)
     @context = context
     @cached_root = nil
-    super(*args)
+
+    options.nil? ? super() : super(**options)
   end
 
   # Generates JSON using the template specified with the `:partial` option. For example, the code below will render
@@ -52,7 +55,9 @@ class JbuilderTemplate < Jbuilder
     if args.one? && _is_active_model?(args.first)
       _render_active_model_partial args.first
     else
-      _render_explicit_partial(*args)
+      options = args.extract_options!.dup
+      options[:partial] = args.first if args.present?
+      _render_partial_with_options options
     end
   end
 
@@ -118,7 +123,9 @@ class JbuilderTemplate < Jbuilder
     options = args.first
 
     if args.one? && _partial_options?(options)
-      partial! options.merge(collection: collection)
+      options = options.dup
+      options[:collection] = collection
+      _render_partial_with_options options
     else
       super
     end
@@ -128,7 +135,7 @@ class JbuilderTemplate < Jbuilder
     options = args.first
 
     if args.one? && _partial_options?(options)
-      _set_inline_partial name, object, options
+      _set_inline_partial name, object, options.dup
     else
       super
     end
@@ -136,15 +143,17 @@ class JbuilderTemplate < Jbuilder
 
   private
 
+  alias_method :method_missing, :set!
+
   def _render_partial_with_options(options)
-    options.reverse_merge! locals: options.except(:partial, :as, :collection, :cached)
-    options.reverse_merge! ::JbuilderTemplate.template_lookup_options
+    options[:locals] ||= options.except(:partial, :as, :collection, :cached)
+    options[:handlers] ||= ::JbuilderTemplate.template_lookup_options[:handlers]
     as = options[:as]
 
-    if as && options.key?(:collection) && CollectionRenderer.supported?
+    if as && options.key?(:collection)
       collection = options.delete(:collection) || []
       partial = options.delete(:partial)
-      options[:locals].merge!(json: self)
+      options[:locals][:json] = self
       collection = EnumerableCompat.new(collection) if collection.respond_to?(:count) && !collection.respond_to?(:size)
 
       if options.has_key?(:layout)
@@ -155,21 +164,14 @@ class JbuilderTemplate < Jbuilder
         ::Kernel.raise ::NotImplementedError, "The `:spacer_template' option is not supported in collection rendering."
       end
 
-      results = CollectionRenderer
-        .new(@context.lookup_context, options) { |&block| _scope(&block) }
-        .render_collection_with_partial(collection, partial, @context, nil)
+      if collection.present?
+        results = CollectionRenderer
+          .new(@context.lookup_context, options) { |&block| _scope(&block) }
+          .render_collection_with_partial(collection, partial, @context, nil)
 
-      array! if results.respond_to?(:body) && results.body.nil?
-    elsif as && options.key?(:collection) && !CollectionRenderer.supported?
-      # For Rails <= 5.2:
-      as = as.to_sym
-      collection = options.delete(:collection)
-      locals = options.delete(:locals)
-      array! collection do |member|
-        member_locals = locals.clone
-        member_locals.merge! collection: collection
-        member_locals.merge! as => member
-        _render_partial options.merge(locals: member_locals)
+        array! if results.respond_to?(:body) && results.body.nil?
+      else
+        array!
       end
     else
       _render_partial options
@@ -177,7 +179,7 @@ class JbuilderTemplate < Jbuilder
   end
 
   def _render_partial(options)
-    options[:locals].merge! json: self
+    options[:locals][:json] = self
     @context.render options
   end
 
@@ -233,34 +235,18 @@ class JbuilderTemplate < Jbuilder
     value = if object.nil?
       []
     elsif _is_collection?(object)
-      _scope{ _render_partial_with_options options.merge(collection: object) }
+      _scope do
+        options[:collection] = object
+        _render_partial_with_options options
+      end
     else
-      locals = ::Hash[options[:as], object]
-      _scope{ _render_partial_with_options options.merge(locals: locals) }
-    end
-
-    set! name, value
-  end
-
-  def _render_explicit_partial(name_or_options, locals = {})
-    case name_or_options
-    when ::Hash
-      # partial! partial: 'name', foo: 'bar'
-      options = name_or_options
-    else
-      # partial! 'name', locals: {foo: 'bar'}
-      if locals.one? && (locals.keys.first == :locals)
-        options = locals.merge(partial: name_or_options)
-      else
-        options = { partial: name_or_options, locals: locals }
+      _scope do
+        options[:locals] = { options[:as] => object }
+        _render_partial_with_options options
       end
-      # partial! 'name', foo: 'bar'
-      as = locals.delete(:as)
-      options[:as] = as if as.present?
-      options[:collection] = locals[:collection] if locals.key?(:collection)
     end
 
-    _render_partial_with_options options
+    _set_value name, value
   end
 
   def _render_active_model_partial(object)
@@ -275,7 +261,7 @@ class JbuilderHandler
   def self.call(template, source = nil)
     source ||= template.source
     # this juggling is required to keep line numbers right in the error
-    %{__already_defined = defined?(json); json||=JbuilderTemplate.new(self); #{source}
+    %{__already_defined = defined?(json); json||=JbuilderTemplate.new(self); #{source};
       json.target! unless (__already_defined && __already_defined != "method")}
   end
 end
